﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;

namespace ExtraltodeusPlugin {
	public class CameraRigPlugin : MVRScript {

		#region PluginInfo    
        public string pluginAuthor = "ky1001";
        public string pluginName = "HeadLight Link";
        public string pluginVersion = "2.2";
        public string pluginDate = "2021-08-24";
        public string pluginDescription = "Headlight.csを元にParent Link出来るように改造しました";
        #endregion

        private UIDynamicToggle ptoggle;
		private UIDynamicToggle ltoggle;
		private JSONStorableStringChooser Target;
		private UIDynamicPopup udp;
        private UIDynamicButton button;
		private UIDynamicSlider sX;
		private UIDynamicSlider sY;
		private UIDynamicSlider sZ;
		private UIDynamicSlider sXa;
		private UIDynamicSlider sYa;
		private UIDynamicSlider sZa;
	
		private JSONStorableBool playingBool;	
		private JSONStorableBool linkBool;
		private bool linkBoolval;

		private JSONStorableFloat X;
		private JSONStorableFloat Y;
		private JSONStorableFloat Z;

		private JSONStorableFloat Xa;
		private JSONStorableFloat Ya;
		private JSONStorableFloat Za;
        Rigidbody linkToAtom;
		private string currentTarget;

		public override void Init() {
			try
			{
				X = new JSONStorableFloat("Position X adjustment", 0f, -1.0f, 1.0f, true);
				X.storeType = JSONStorableParam.StoreType.Full;
				sX = CreateSlider(X);
				RegisterFloat(X);
				sX.slider.onValueChanged.AddListener(delegate { linkBoolval = false; ChangeLink(); });

				Y = new JSONStorableFloat("Position Y adjustment", 0f, -1.0f, 1.0f, true);
				Y.storeType = JSONStorableParam.StoreType.Full;
				sY = CreateSlider(Y);
				RegisterFloat(Y);
				sY.slider.onValueChanged.AddListener(delegate { linkBoolval = false; ChangeLink(); });

				Z = new JSONStorableFloat("Position Z adjustment", 0f, -1.0f, 1.0f, true);
				Z.storeType = JSONStorableParam.StoreType.Full;
				sZ = CreateSlider(Z);
				RegisterFloat(Z);
				sZ.slider.onValueChanged.AddListener(delegate { linkBoolval = false; ChangeLink(); });

				Xa = new JSONStorableFloat("Rotation X adjustment", 0f, -2.0f, 2.0f, true);
				Xa.storeType = JSONStorableParam.StoreType.Full;
				sXa = CreateSlider(Xa);
				RegisterFloat(Xa);
				sXa.slider.onValueChanged.AddListener(delegate { linkBoolval = false; ChangeLink(); });

				Ya = new JSONStorableFloat("Rotation Y adjustment", 0f, -2.0f, 2.0f, true);
				Ya.storeType = JSONStorableParam.StoreType.Full;
				sYa = CreateSlider(Ya);
				RegisterFloat(Ya);
				sYa.slider.onValueChanged.AddListener(delegate { linkBoolval = false; ChangeLink(); });

				Za = new JSONStorableFloat("Rotation Z adjustment", 0f, -2.0f, 2.0f, true);
				Za.storeType = JSONStorableParam.StoreType.Full;
				sZa = CreateSlider(Za);
				RegisterFloat(Za);
				sZa.slider.onValueChanged.AddListener(delegate { linkBoolval = false; ChangeLink(); });

				playingBool = new JSONStorableBool("Play", true);
				playingBool.storeType = JSONStorableParam.StoreType.Full;
				RegisterBool(playingBool);
				ptoggle = CreateToggle((playingBool), true);

				List<string> targetChoices = new List<string>();
				targetChoices.Add("CenterEye");
				targetChoices.Add("LeftHand");
				targetChoices.Add("RightHand");

				Target = new JSONStorableStringChooser("Target", targetChoices, "CenterEye", "Choose Target");
				Target.storeType = JSONStorableParam.StoreType.Full;
				RegisterStringChooser(Target);
				udp = CreatePopup(Target, true);
				//currentTarget = Target.val;
				
				linkBool = new JSONStorableBool("Parent Link", true);
				linkBool.storeType = JSONStorableParam.StoreType.Full;
				RegisterBool(linkBool);
				ltoggle = CreateToggle((linkBool), true);
				ltoggle.toggle.onValueChanged.AddListener(delegate { ChangeLink(); });

				//linkBool = new JSONStorableBool("dummy", true);
				//linkBool.storeType = JSONStorableParam.StoreType.Full;
				//RegisterBool(linkBool);
				//ltoggle = CreateToggle((linkBool), true);
				//ltoggle.toggle.onValueChanged.AddListener(delegate { ChangeLink(); });
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		void Start() {
			linkBoolval = false;
			ChangeLink();
		}

		void FixedUpdate() {
			try
			{
				if (ptoggle.toggle.isOn)
				{
					if (Target.val == "CenterEye")
					{
						Vector3 pos = SuperController.singleton.lookCamera.transform.position;
						Vector3 rot = SuperController.singleton.lookCamera.transform.eulerAngles;

						pos[0] = X.val;
						pos[1] = Y.val;
						pos[2] = Z.val;

						rot[0] = rot[0] + Xa.val*90f;
						rot[1] = rot[1] + Ya.val*90f;
						rot[2] = rot[2] + Za.val*90f;

						pos = SuperController.singleton.lookCamera.transform.TransformPoint(pos);

						containingAtom.GetStorableByID("control").transform.position    = pos;
						containingAtom.GetStorableByID("control").transform.eulerAngles = rot;
					}
					else if (Target.val == "LeftHand")
					{
						Vector3 pos = SuperController.singleton.leftHand.transform.position;
						Vector3 rot = SuperController.singleton.leftHand.transform.eulerAngles;

						pos[0] = X.val;
						pos[1] = Y.val;
						pos[2] = Z.val;

						rot[0] = rot[0] + Xa.val*90f;
						rot[1] = rot[1] + Ya.val*90f;
						rot[2] = rot[2] + Za.val*90f;

						pos = SuperController.singleton.leftHand.transform.TransformPoint(pos);

						containingAtom.GetStorableByID("control").transform.position    = pos;
						containingAtom.GetStorableByID("control").transform.eulerAngles = rot;
					}
					else if (Target.val == "RightHand")
					{
						Vector3 pos = SuperController.singleton.rightHand.transform.position;
						Vector3 rot = SuperController.singleton.rightHand.transform.eulerAngles;

						pos[0] = X.val;
						pos[1] = Y.val;
						pos[2] = Z.val;

						rot[0] = rot[0] + Xa.val*90f;
						rot[1] = rot[1] + Ya.val*90f;
						rot[2] = rot[2] + Za.val*90f;

						pos = SuperController.singleton.rightHand.transform.TransformPoint(pos);

						containingAtom.GetStorableByID("control").transform.position    = pos;
						containingAtom.GetStorableByID("control").transform.eulerAngles = rot;
					}
				}
				else
				{
					linkBool.val = false;
				}

				if (linkBool.val == false)
				{
					linkBoolval = false;
					ChangeLink();					
				}
				else
				{
					linkBoolval = true;
					ChangeLink();					
				}
								
				if (currentTarget != Target.val)
				{
					currentTarget = Target.val;
					//SuperController.LogMessage("Target Change");
					ChangeLink();
				}
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		void ChangeLink()
		{
			if (linkBoolval)
			{
				ChangeTarget();
			}
			else
			{
				StopLink();
			}
		}

		void ChangeTarget()
		{
			FreeControllerV3 Control = containingAtom.GetStorableByID("control") as FreeControllerV3;
			Control.currentPositionState = FreeControllerV3.PositionState.ParentLink;
			Control.currentRotationState = FreeControllerV3.RotationState.ParentLink;
			linkToAtom = SuperController.singleton.RigidbodyNameToRigidbody("[CameraRig]:" + Target.val);
			Control.SelectLinkToRigidbody(linkToAtom, FreeControllerV3.SelectLinkState.PositionAndRotation);

			//SuperController.LogMessage(currentTarget);
		}

		void StopLink()
		{
			FreeControllerV3 Control = containingAtom.GetStorableByID("control") as FreeControllerV3;
			Control.currentPositionState = FreeControllerV3.PositionState.On;
			Control.currentRotationState = FreeControllerV3.RotationState.On;
			linkToAtom = SuperController.singleton.RigidbodyNameToRigidbody("None:None");
			Control.SelectLinkToRigidbody(linkToAtom, FreeControllerV3.SelectLinkState.PositionAndRotation);

			//SuperController.LogMessage("Link Off");
		}
	}
}
